
From: Michael Krufky <mkrufky@m1k.net>

Remove the dvb_pll_desc from the frontend and replace with a
pll_set-callback to isolate the tuner programming from the frontend.

Signed-off-by: Mac Michaels <wmichaels1@earthlink.net>
Signed-off-by: Michael Krufky <mkrufky@m1k.net>
Signed-off-by: Andrew Morton <akpm@osdl.org>
---

 drivers/media/dvb/frontends/lgdt3302.c |   68 +++++++++++++++------------------
 drivers/media/dvb/frontends/lgdt3302.h |    7 +--
 drivers/media/video/cx88/cx88-dvb.c    |   34 ++++++++++------
 3 files changed, 55 insertions(+), 54 deletions(-)

diff -puN drivers/media/dvb/frontends/lgdt3302.c~dvb-v4l-lgdt3302-isolate-tuner drivers/media/dvb/frontends/lgdt3302.c
--- devel/drivers/media/dvb/frontends/lgdt3302.c~dvb-v4l-lgdt3302-isolate-tuner	2005-07-25 21:51:25.000000000 -0700
+++ devel-akpm/drivers/media/dvb/frontends/lgdt3302.c	2005-07-25 21:51:25.000000000 -0700
@@ -74,11 +74,14 @@ static int i2c_writebytes (struct lgdt33
 			   u8 *buf, /* data bytes to send */
 			   int len  /* number of bytes to send */ )
 {
-	if (addr == state->config->pll_address) {
-		struct i2c_msg msg =
-			{ .addr = addr, .flags = 0, .buf = buf,  .len = len };
-		int err;
+	u8 tmp[] = { buf[0], buf[1] };
+	struct i2c_msg msg =
+		{ .addr = addr, .flags = 0, .buf = tmp,  .len = 2 };
+	int err;
+	int i;
 
+	for (i=1; i<len; i++) {
+		tmp[1] = buf[i];
 		if ((err = i2c_transfer(state->i2c, &msg, 1)) != 1) {
 			printk(KERN_WARNING "lgdt3302: %s error (addr %02x <- %02x, err == %i)\n", __FUNCTION__, addr, buf[0], err);
 			if (err < 0)
@@ -86,27 +89,11 @@ static int i2c_writebytes (struct lgdt33
 			else
 				return -EREMOTEIO;
 		}
-	} else {
-		u8 tmp[] = { buf[0], buf[1] };
-		struct i2c_msg msg =
-			{ .addr = addr, .flags = 0, .buf = tmp,  .len = 2 };
-		int err;
-		int i;
-
-		for (i=1; i<len; i++) {
-			tmp[1] = buf[i];
-			if ((err = i2c_transfer(state->i2c, &msg, 1)) != 1) {
-				printk(KERN_WARNING "lgdt3302: %s error (addr %02x <- %02x, err == %i)\n", __FUNCTION__, addr, buf[0], err);
-				if (err < 0)
-					return err;
-				else
-					return -EREMOTEIO;
-			}
-			tmp[0]++;
-		}
+		tmp[0]++;
 	}
 	return 0;
 }
+
 static int i2c_readbytes (struct lgdt3302_state* state,
 			  u8 addr, /* demod_address or pll_address */
 			  u8 *buf, /* holds data bytes read */
@@ -207,7 +194,6 @@ static int lgdt3302_read_ucblocks(struct
 static int lgdt3302_set_parameters(struct dvb_frontend* fe,
 				   struct dvb_frontend_parameters *param)
 {
-	u8 buf[4];
 	struct lgdt3302_state* state =
 		(struct lgdt3302_state*) fe->demodulator_priv;
 
@@ -290,16 +276,30 @@ static int lgdt3302_set_parameters(struc
 
 	/* Change only if we are actually changing the channel */
 	if (state->current_frequency != param->frequency) {
-		dvb_pll_configure(state->config->pll_desc, buf,
-				  param->frequency, 0);
-		dprintk("%s: tuner bytes: 0x%02x 0x%02x "
-			"0x%02x 0x%02x\n", __FUNCTION__, buf[0],buf[1],buf[2],buf[3]);
-		i2c_writebytes(state, state->config->pll_address ,buf, 4);
+		u8 buf[5];
 
-		/* Check the status of the tuner pll */
-		i2c_readbytes(state, state->config->pll_address, buf, 1);
-		dprintk("%s: tuner status byte = 0x%02x\n", __FUNCTION__, buf[0]);
+		/* This must be done before the initialized msg is declared */
+		state->config->pll_set(fe, param, buf);
 
+		struct i2c_msg msg =
+			{ .addr = buf[0], .flags = 0, .buf = &buf[1], .len = 4 };
+		int err;
+
+		dprintk("%s: tuner at 0x%02x bytes: 0x%02x 0x%02x "
+			"0x%02x 0x%02x\n", __FUNCTION__,
+			buf[0],buf[1],buf[2],buf[3],buf[4]);
+		if ((err = i2c_transfer(state->i2c, &msg, 1)) != 1) {
+			printk(KERN_WARNING "lgdt3302: %s error (addr %02x <- %02x, err = %i)\n", __FUNCTION__, buf[0], buf[1], err);
+			if (err < 0)
+				return err;
+			else
+				return -EREMOTEIO;
+		}
+#if 0
+		/* Check the status of the tuner pll */
+		i2c_readbytes(state, buf[0], &buf[1], 1);
+		dprintk("%s: tuner status byte = 0x%02x\n", __FUNCTION__, buf[1]);
+#endif
 		/* Update current frequency */
 		state->current_frequency = param->frequency;
 	}
@@ -322,12 +322,6 @@ static int lgdt3302_read_status(struct d
 
 	*status = 0; /* Reset status result */
 
-	/* Check the status of the tuner pll */
-	i2c_readbytes(state, state->config->pll_address, buf, 1);
-	dprintk("%s: tuner status byte = 0x%02x\n", __FUNCTION__, buf[0]);
-	if ((buf[0] & 0xc0) != 0x40)
-		return 0; /* Tuner PLL not locked or not powered on */
-
 	/*
 	 * You must set the Mask bits to 1 in the IRQ_MASK in order
 	 * to see that status bit in the IRQ_STATUS register.
diff -puN drivers/media/dvb/frontends/lgdt3302.h~dvb-v4l-lgdt3302-isolate-tuner drivers/media/dvb/frontends/lgdt3302.h
--- devel/drivers/media/dvb/frontends/lgdt3302.h~dvb-v4l-lgdt3302-isolate-tuner	2005-07-25 21:51:25.000000000 -0700
+++ devel-akpm/drivers/media/dvb/frontends/lgdt3302.h	2005-07-25 21:51:25.000000000 -0700
@@ -1,6 +1,4 @@
 /*
- * $Id: lgdt3302.h,v 1.2 2005/06/28 23:50:48 mkrufky Exp $
- *
  *    Support for LGDT3302 (DViCO FustionHDTV 3 Gold) - VSB/QAM
  *
  *    Copyright (C) 2005 Wilson Michaels <wilsonmichaels@earthlink.net>
@@ -30,8 +28,9 @@ struct lgdt3302_config
 {
 	/* The demodulator's i2c address */
 	u8 demod_address;
-	u8 pll_address;
-	struct dvb_pll_desc *pll_desc;
+
+	/* PLL interface */
+	int (*pll_set)(struct dvb_frontend* fe, struct dvb_frontend_parameters* params, u8* pll_address);
 
 	/* Need to set device param for start_dma */
 	int (*set_ts_params)(struct dvb_frontend* fe, int is_punctured);
diff -puN drivers/media/video/cx88/cx88-dvb.c~dvb-v4l-lgdt3302-isolate-tuner drivers/media/video/cx88/cx88-dvb.c
--- devel/drivers/media/video/cx88/cx88-dvb.c~dvb-v4l-lgdt3302-isolate-tuner	2005-07-25 21:51:25.000000000 -0700
+++ devel-akpm/drivers/media/video/cx88/cx88-dvb.c	2005-07-25 21:51:25.000000000 -0700
@@ -1,5 +1,5 @@
 /*
- * $Id: cx88-dvb.c,v 1.42 2005/07/12 15:44:55 mkrufky Exp $
+ * $Id: cx88-dvb.c,v 1.47 2005/07/20 05:20:37 mkrufky Exp $
  *
  * device driver for Conexant 2388x based TV cards
  * MPEG Transport Stream (DVB) routines
@@ -211,6 +211,18 @@ static struct or51132_config pchdtv_hd30
 #endif
 
 #if CONFIG_DVB_LGDT3302
+static int lgdt3302_pll_set(struct dvb_frontend* fe,
+			    struct dvb_frontend_parameters* params,
+			    u8* pllbuf)
+{
+	struct cx8802_dev *dev= fe->dvb->priv;
+
+	pllbuf[0] = dev->core->pll_addr;
+	dvb_pll_configure(dev->core->pll_desc, &pllbuf[1],
+			  params->frequency, 0);
+	return 0;
+}
+
 static int lgdt3302_set_ts_param(struct dvb_frontend* fe, int is_punctured)
 {
 	struct cx8802_dev *dev= fe->dvb->priv;
@@ -221,17 +233,9 @@ static int lgdt3302_set_ts_param(struct 
 	return 0;
 }
 
-static struct lgdt3302_config fusionhdtv_3_gold_q = {
-	.demod_address    = 0x0e,
-	.pll_address      = 0x61,
-	.pll_desc         = &dvb_pll_microtune_4042,
-	.set_ts_params    = lgdt3302_set_ts_param,
-};
-
-static struct lgdt3302_config fusionhdtv_3_gold_t = {
+static struct lgdt3302_config fusionhdtv_3_gold = {
 	.demod_address    = 0x0e,
-	.pll_address      = 0x61,
-	.pll_desc         = &dvb_pll_thomson_dtt7611,
+	.pll_set          = lgdt3302_pll_set,
 	.set_ts_params    = lgdt3302_set_ts_param,
 };
 #endif
@@ -294,7 +298,9 @@ static int dvb_register(struct cx8802_de
 		mdelay(100);
 		cx_set(MO_GP0_IO, 9); // ANT connector too FIXME
 		mdelay(200);
-		dev->dvb.frontend = lgdt3302_attach(&fusionhdtv_3_gold_q,
+		dev->core->pll_addr = 0x61;
+		dev->core->pll_desc = &dvb_pll_microtune_4042;
+		dev->dvb.frontend = lgdt3302_attach(&fusionhdtv_3_gold,
 						    &dev->core->i2c_adap);
 		}
 		break;
@@ -308,7 +314,9 @@ static int dvb_register(struct cx8802_de
 		mdelay(100);
 		cx_set(MO_GP0_IO, 9); /* ANT connector too FIXME */
 		mdelay(200);
-		dev->dvb.frontend = lgdt3302_attach(&fusionhdtv_3_gold_t,
+		dev->core->pll_addr = 0x61;
+		dev->core->pll_desc = &dvb_pll_thomson_dtt7611;
+		dev->dvb.frontend = lgdt3302_attach(&fusionhdtv_3_gold,
 						    &dev->core->i2c_adap);
 		}
 		break;
_
